@@ -41,7 +41,8 @@ class UnshardedLegacyPrecomputedMeshSource(object):
       self.spatial_index = CachedSpatialIndex(
         self.cache, self.config,
         cloudpath=self.meta.layerpath, 
-        bounds=self.meta.meta.bounds(mip) * self.meta.meta.resolution(mip),
+        bounds=self.meta.meta.bounds(mip), 
+        resolution=self.meta.info['spatial_index'].get('resolution', self.meta.meta.resolution(mip)),
         chunk_size=self.meta.info['spatial_index']['chunk_size'],
       )
 
